@@ -2,7 +2,7 @@
  * Bittorrent Client using Qt and libtorrent.
  * Copyright (C) 2016  Eugene Shalygin <eugene.shalygin@gmail.com>
  * Copyright (C) 2014  Vladimir Golovnev <glassez@yandex.ru>
- * Copyright (C) 2006  Christophe Dumez
+ * Copyright (C) 2006  Christophe Dumez <chris@qbittorrent.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,13 +26,11 @@
  * modify file(s), you may extend this exception to your version of the file(s),
  * but you are not obligated to do so. If you do not wish to do so, delete this
  * exception statement from your version.
- *
- * Contact : chris@qbittorrent.org
  */
 
 #include "cmdoptions.h"
 
-#include <iostream>
+#include <cstdio>
 
 #include <QDebug>
 #include <QFileInfo>
@@ -100,7 +98,7 @@ namespace
     };
 
     // Boolean option.
-    class BoolOption: protected Option
+    class BoolOption : protected Option
     {
     public:
         constexpr BoolOption(const char *name, char shortcut = 0)
@@ -118,7 +116,7 @@ namespace
         {
             QString val = env.value(envVarName());
             // we accept "1" and "true" (upper or lower cased) as boolean 'true' values
-            return (val == QLatin1String("1") || val.toUpper() == QLatin1String("TRUE"));
+            return ((val == QLatin1String("1")) || (val.toUpper() == QLatin1String("TRUE")));
         }
 
         QString usage() const
@@ -137,7 +135,7 @@ namespace
     }
 
     // Option with string value. May not have a shortcut
-    struct StringOption: protected Option
+    struct StringOption : protected Option
     {
     public:
         constexpr StringOption(const char *name)
@@ -184,7 +182,7 @@ namespace
     }
 
     // Option with integer value. May not have a shortcut
-    class IntOption: protected StringOption
+    class IntOption : protected StringOption
     {
     public:
         constexpr IntOption(const char *name)
@@ -216,7 +214,7 @@ namespace
             int res = val.toInt(&ok);
             if (!ok) {
                 qDebug() << QObject::tr("Expected integer number in environment variable '%1', but got '%2'")
-                    .arg(envVarName()).arg(val);
+                    .arg(envVarName(), val);
                 return defaultValue;
             }
             return res;
@@ -230,7 +228,7 @@ namespace
 
     // Option that is explicitly set to true or false, and whose value is undefined when unspecified.
     // May not have a shortcut.
-    class TriStateBoolOption: protected Option
+    class TriStateBoolOption : protected Option
     {
     public:
         constexpr TriStateBoolOption(const char *name, bool defaultValue)
@@ -260,10 +258,10 @@ namespace
             else if (parts.size() == 2) {
                 QString val = parts[1];
 
-                if (val.toUpper() == QLatin1String("TRUE") || val == QLatin1String("1")) {
+                if ((val.toUpper() == QLatin1String("TRUE")) || (val == QLatin1String("1"))) {
                     return TriStateBool::True;
                 }
-                else if (val.toUpper() == QLatin1String("FALSE") || val == QLatin1String("0")) {
+                else if ((val.toUpper() == QLatin1String("FALSE")) || (val == QLatin1String("0"))) {
                     return TriStateBool::False;
                 }
             }
@@ -285,15 +283,15 @@ namespace
             else if (val == QLatin1String("-1")) {
                 return TriStateBool::Undefined;
             }
-            else if (val.toUpper() == QLatin1String("TRUE") || val == QLatin1String("1")) {
+            else if ((val.toUpper() == QLatin1String("TRUE")) || (val == QLatin1String("1"))) {
                 return TriStateBool::True;
             }
-            else if (val.toUpper() == QLatin1String("FALSE") || val == QLatin1String("0")) {
+            else if ((val.toUpper() == QLatin1String("FALSE")) || (val == QLatin1String("0"))) {
                 return TriStateBool::False;
             }
             else {
                 qDebug() << QObject::tr("Expected %1 in environment variable '%2', but got '%3'")
-                    .arg(QLatin1String("true|false")).arg(envVarName()).arg(val);
+                    .arg(QLatin1String("true|false"), envVarName(), val);
                 return TriStateBool::Undefined;
             }
         }
@@ -360,7 +358,7 @@ QStringList QBtCommandLineParameters::paramList() const
     // the user has specified. Here we place special strings that are
     // almost certainly not going to collide with a file path or URL
     // specified by the user, and placing them at the beginning of the
-    // string listr so that they will be processed before the list of
+    // string list so that they will be processed before the list of
     // torrent paths or URLs.
 
     if (!savePath.isEmpty())
@@ -404,9 +402,9 @@ QBtCommandLineParameters parseCommandLine(const QStringList &args)
         const QString &arg = args[i];
 
         if ((arg.startsWith("--") && !arg.endsWith(".torrent"))
-            || (arg.startsWith("-") && (arg.size() == 2))) {
+            || (arg.startsWith('-') && (arg.size() == 2))) {
             // Parse known parameters
-            if ((arg == SHOW_HELP_OPTION)) {
+            if (arg == SHOW_HELP_OPTION) {
                 result.showHelp = true;
             }
 #ifndef Q_OS_WIN
@@ -501,7 +499,7 @@ QString wrapText(const QString &text, int initialIndentation = USAGE_TEXT_COLUMN
 
     foreach (const QString &word, words.mid(1)) {
         if (lines.last().length() + word.length() + 1 < currentLineMaxLength) {
-            lines.last().append(" " + word);
+            lines.last().append(' ' + word);
         }
         else {
             lines.append(QString(initialIndentation, ' ') + word);
@@ -509,7 +507,7 @@ QString wrapText(const QString &text, int initialIndentation = USAGE_TEXT_COLUMN
         }
     }
 
-    return lines.join("\n");
+    return lines.join('\n');
 }
 
 QString makeUsage(const QString &prgName)
@@ -578,7 +576,7 @@ QString makeUsage(const QString &prgName)
 void displayUsage(const QString &prgName)
 {
 #ifndef Q_OS_WIN
-    std::cout << qPrintable(makeUsage(prgName)) << std::endl;
+    printf("%s\n", qUtf8Printable(makeUsage(prgName)));
 #else
     QMessageBox msgBox(QMessageBox::Information, QObject::tr("Help"), makeUsage(prgName), QMessageBox::Ok);
     msgBox.show(); // Need to be shown or to moveToCenter does not work
