@@ -1,5 +1,5 @@
 /*
- * Bittorrent Client using Qt4 and libtorrent.
+ * Bittorrent Client using Qt and libtorrent.
  * Copyright (C) 2015  Vladimir Golovnev <glassez@yandex.ru>
  * Copyright (C) 2006  Christophe Dumez <chris@qbittorrent.org>
  *
@@ -27,15 +27,18 @@
  * exception statement from your version.
  */
 
+#include "tracker.h"
+
 #include <vector>
 
 #include <libtorrent/bencode.hpp>
 #include <libtorrent/entry.hpp>
 
+#include "base/global.h"
 #include "base/http/server.h"
 #include "base/preferences.h"
+#include "base/utils/bytearray.h"
 #include "base/utils/string.h"
-#include "tracker.h"
 
 // static limits
 static const int MAX_TORRENTS = 100;
@@ -57,7 +60,7 @@ bool Peer::operator==(const Peer &other) const
 
 QString Peer::uid() const
 {
-    return ip + ":" + QString::number(port);
+    return ip + ':' + QString::number(port);
 }
 
 libtorrent::entry Peer::toEntry(bool noPeerId) const
@@ -74,7 +77,7 @@ libtorrent::entry Peer::toEntry(bool noPeerId) const
 // Tracker
 
 Tracker::Tracker(QObject *parent)
-    : Http::ResponseBuilder(parent)
+    : QObject(parent)
     , m_server(new Http::Server(this, this))
 {
 }
@@ -130,19 +133,30 @@ Http::Response Tracker::processRequest(const Http::Request &request, const Http:
 
 void Tracker::respondToAnnounceRequest()
 {
-    const QStringMap &gets = m_request.gets;
+    QMap<QString, QByteArray> queryParams;
+    // Parse GET parameters
+    using namespace Utils::ByteArray;
+    for (const QByteArray &param : copyAsConst(splitToViews(m_request.query, "&"))) {
+        const int sepPos = param.indexOf('=');
+        if (sepPos <= 0) continue; // ignores params without name
+
+        const QString paramName {QString::fromUtf8(param.constData(), sepPos)};
+        const QByteArray paramValue {param.mid(sepPos + 1)};
+        queryParams[paramName] = paramValue;
+    }
+
     TrackerAnnounceRequest annonceReq;
 
     // IP
     annonceReq.peer.ip = m_env.clientAddress.toString();
 
     // 1. Get info_hash
-    if (!gets.contains("info_hash")) {
+    if (!queryParams.contains("info_hash")) {
         qDebug("Tracker: Missing info_hash");
         status(101, "Missing info_hash");
         return;
     }
-    annonceReq.infoHash = gets.value("info_hash");
+    annonceReq.infoHash = queryParams.value("info_hash");
     // info_hash cannot be longer than 20 bytes
     /*if (annonce_req.info_hash.toLatin1().length() > 20) {
         qDebug("Tracker: Info_hash is not 20 byte long: %s (%d)", qUtf8Printable(annonce_req.info_hash), annonce_req.info_hash.toLatin1().length());
@@ -151,12 +165,12 @@ void Tracker::respondToAnnounceRequest()
       }*/
 
     // 2. Get peer ID
-    if (!gets.contains("peer_id")) {
+    if (!queryParams.contains("peer_id")) {
         qDebug("Tracker: Missing peer_id");
         status(102, "Missing peer_id");
         return;
     }
-    annonceReq.peer.peerId = gets.value("peer_id");
+    annonceReq.peer.peerId = queryParams.value("peer_id");
     // peer_id cannot be longer than 20 bytes
     /*if (annonce_req.peer.peer_id.length() > 20) {
         qDebug("Tracker: peer_id is not 20 byte long: %s", qUtf8Printable(annonce_req.peer.peer_id));
@@ -165,14 +179,14 @@ void Tracker::respondToAnnounceRequest()
       }*/
 
     // 3. Get port
-    if (!gets.contains("port")) {
+    if (!queryParams.contains("port")) {
         qDebug("Tracker: Missing port");
         status(103, "Missing port");
         return;
     }
     bool ok = false;
-    annonceReq.peer.port = gets.value("port").toInt(&ok);
-    if (!ok || annonceReq.peer.port < 1 || annonceReq.peer.port > 65535) {
+    annonceReq.peer.port = queryParams.value("port").toInt(&ok);
+    if (!ok || (annonceReq.peer.port < 0) || (annonceReq.peer.port > 65535)) {
         qDebug("Tracker: Invalid port number (%d)", annonceReq.peer.port);
         status(103, "Missing port");
         return;
@@ -180,15 +194,15 @@ void Tracker::respondToAnnounceRequest()
 
     // 4.  Get event
     annonceReq.event = "";
-    if (gets.contains("event")) {
-        annonceReq.event = gets.value("event");
+    if (queryParams.contains("event")) {
+        annonceReq.event = queryParams.value("event");
         qDebug("Tracker: event is %s", qUtf8Printable(annonceReq.event));
     }
 
     // 5. Get numwant
     annonceReq.numwant = 50;
-    if (gets.contains("numwant")) {
-        int tmp = gets.value("numwant").toInt();
+    if (queryParams.contains("numwant")) {
+        int tmp = queryParams.value("numwant").toInt();
         if (tmp > 0) {
             qDebug("Tracker: numwant = %d", tmp);
             annonceReq.numwant = tmp;
@@ -197,37 +211,51 @@ void Tracker::respondToAnnounceRequest()
 
     // 6. no_peer_id (extension)
     annonceReq.noPeerId = false;
-    if (gets.contains("no_peer_id"))
+    if (queryParams.contains("no_peer_id"))
         annonceReq.noPeerId = true;
 
     // 7. TODO: support "compact" extension
 
     // Done parsing, now let's reply
-    if (m_torrents.contains(annonceReq.infoHash)) {
-        if (annonceReq.event == "stopped") {
-            qDebug("Tracker: Peer stopped downloading, deleting it from the list");
-            m_torrents[annonceReq.infoHash].remove(annonceReq.peer.uid());
-            return;
-        }
+    if (annonceReq.event == "stopped") {
+        unregisterPeer(annonceReq);
     }
     else {
+        registerPeer(annonceReq);
+        replyWithPeerList(annonceReq);
+    }
+}
+
+void Tracker::registerPeer(const TrackerAnnounceRequest &annonceReq)
+{
+    if (annonceReq.peer.port == 0) return;
+
+    if (!m_torrents.contains(annonceReq.infoHash)) {
         // Unknown torrent
         if (m_torrents.size() == MAX_TORRENTS) {
             // Reached max size, remove a random torrent
             m_torrents.erase(m_torrents.begin());
         }
     }
+
     // Register the user
-    PeerList peers = m_torrents.value(annonceReq.infoHash);
-    if (peers.size() == MAX_PEERS_PER_TORRENT) {
-        // Too many peers, remove a random one
-        peers.erase(peers.begin());
+    PeerList &peers = m_torrents[annonceReq.infoHash];
+    if (!peers.contains(annonceReq.peer.uid())) {
+        // Unknown peer
+        if (peers.size() == MAX_PEERS_PER_TORRENT) {
+            // Too many peers, remove a random one
+            peers.erase(peers.begin());
+        }
     }
     peers[annonceReq.peer.uid()] = annonceReq.peer;
-    m_torrents[annonceReq.infoHash] = peers;
+}
 
-    // Reply
-    replyWithPeerList(annonceReq);
+void Tracker::unregisterPeer(const TrackerAnnounceRequest &annonceReq)
+{
+    if (annonceReq.peer.port == 0) return;
+
+    if (m_torrents[annonceReq.infoHash].remove(annonceReq.peer.uid()) > 0)
+        qDebug("Tracker: Peer stopped downloading, deleting it from the list");
 }
 
 void Tracker::replyWithPeerList(const TrackerAnnounceRequest &annonceReq)
@@ -235,22 +263,18 @@ void Tracker::replyWithPeerList(const TrackerAnnounceRequest &annonceReq)
     // Prepare the entry for bencoding
     libtorrent::entry::dictionary_type replyDict;
     replyDict["interval"] = libtorrent::entry(ANNOUNCE_INTERVAL);
-    QList<Peer> peers = m_torrents.value(annonceReq.infoHash).values();
+
     libtorrent::entry::list_type peerList;
-    foreach (const Peer &p, peers) {
-        //if (p != annonce_req.peer)
+    for (const Peer &p : m_torrents.value(annonceReq.infoHash))
         peerList.push_back(p.toEntry(annonceReq.noPeerId));
-    }
     replyDict["peers"] = libtorrent::entry(peerList);
-    libtorrent::entry replyEntry(replyDict);
+
+    const libtorrent::entry replyEntry(replyDict);
     // bencode
-    std::vector<char> buf;
-    libtorrent::bencode(std::back_inserter(buf), replyEntry);
-    QByteArray reply(&buf[0], static_cast<int>(buf.size()));
+    QByteArray reply;
+    libtorrent::bencode(std::back_inserter(reply), replyEntry);
     qDebug("Tracker: reply with the following bencoded data:\n %s", reply.constData());
 
     // HTTP reply
     print(reply, Http::CONTENT_TYPE_TXT);
 }
-
-
