@@ -1,5 +1,6 @@
 /*
  * Bittorrent Client using Qt and libtorrent.
+ * Copyright (C) 2018  Mike Tzou (Chocobo1)
  * Copyright (C) 2014  Vladimir Golovnev <glassez@yandex.ru>
  *
  * This program is free software; you can redistribute it and/or
@@ -37,7 +38,12 @@
 
 namespace Http
 {
+    const char METHOD_GET[] = "GET";
+    const char METHOD_POST[] = "POST";
+
     const char HEADER_CACHE_CONTROL[] = "cache-control";
+    const char HEADER_CONNECTION[] = "connection";
+    const char HEADER_CONTENT_DISPOSITION[] = "content-disposition";
     const char HEADER_CONTENT_ENCODING[] = "content-encoding";
     const char HEADER_CONTENT_LENGTH[] = "content-length";
     const char HEADER_CONTENT_SECURITY_POLICY[] = "content-security-policy";
@@ -52,16 +58,21 @@ namespace Http
     const char HEADER_X_FRAME_OPTIONS[] = "x-frame-options";
     const char HEADER_X_XSS_PROTECTION[] = "x-xss-protection";
 
-    const char CONTENT_TYPE_CSS[] = "text/css; charset=UTF-8";
-    const char CONTENT_TYPE_GIF[] = "image/gif";
-    const char CONTENT_TYPE_HTML[] = "text/html; charset=UTF-8";
-    const char CONTENT_TYPE_JS[] = "application/javascript; charset=UTF-8";
-    const char CONTENT_TYPE_JSON[] = "application/json";
-    const char CONTENT_TYPE_PNG[] = "image/png";
-    const char CONTENT_TYPE_TXT[] = "text/plain; charset=UTF-8";
-    const char CONTENT_TYPE_SVG[] = "image/svg+xml";
+    const char HEADER_REQUEST_METHOD_GET[] = "GET";
+    const char HEADER_REQUEST_METHOD_HEAD[] = "HEAD";
+    const char HEADER_REQUEST_METHOD_POST[] = "POST";
 
-    // portability: "\r\n" doesn't guarantee mapping to the correct value
+    const char CONTENT_TYPE_HTML[] = "text/html";
+    const char CONTENT_TYPE_CSS[] = "text/css";
+    const char CONTENT_TYPE_TXT[] = "text/plain";
+    const char CONTENT_TYPE_JS[] = "application/javascript";
+    const char CONTENT_TYPE_JSON[] = "application/json";
+    const char CONTENT_TYPE_GIF[] = "image/gif";
+    const char CONTENT_TYPE_PNG[] = "image/png";
+    const char CONTENT_TYPE_FORM_ENCODED[] = "application/x-www-form-urlencoded";
+    const char CONTENT_TYPE_FORM_DATA[] = "multipart/form-data";
+
+    // portability: "\r\n" doesn't guarantee mapping to the correct symbol
     const char CRLF[] = {0x0D, 0x0A, '\0'};
 
     struct Environment
@@ -75,17 +86,18 @@ namespace Http
 
     struct UploadedFile
     {
-        QString filename; // original filename
-        QString type; // MIME type
-        QByteArray data; // File data
+        QString filename;
+        QString type;  // MIME type
+        QByteArray data;
     };
 
     struct Request
     {
+        QString version;
         QString method;
         QString path;
+        QByteArray query;
         QStringMap headers;
-        QStringMap gets;
         QStringMap posts;
         QVector<UploadedFile> files;
     };
