@@ -34,18 +34,19 @@
 #include <QHash>
 #include <QJsonArray>
 #include <QJsonObject>
-#include <QRegExp>
 #include <QRegularExpression>
 #include <QSharedData>
 #include <QString>
 #include <QStringList>
 
+#include "../global.h"
 #include "../preferences.h"
 #include "../tristatebool.h"
 #include "../utils/fs.h"
 #include "../utils/string.h"
-#include "rss_feed.h"
 #include "rss_article.h"
+#include "rss_autodownloader.h"
+#include "rss_feed.h"
 
 namespace
 {
@@ -100,10 +101,12 @@ const QString Str_AssignedCategory(QStringLiteral("assignedCategory"));
 const QString Str_LastMatch(QStringLiteral("lastMatch"));
 const QString Str_IgnoreDays(QStringLiteral("ignoreDays"));
 const QString Str_AddPaused(QStringLiteral("addPaused"));
+const QString Str_SmartFilter(QStringLiteral("smartFilter"));
+const QString Str_PreviouslyMatched(QStringLiteral("previouslyMatchedEpisodes"));
 
 namespace RSS
 {
-    struct AutoDownloadRuleData: public QSharedData
+    struct AutoDownloadRuleData : public QSharedData
     {
         QString name;
         bool enabled = true;
@@ -120,6 +123,10 @@ namespace RSS
         QString category;
         TriStateBool addPaused = TriStateBool::Undefined;
 
+        bool smartFilter = false;
+        QStringList previouslyMatchedEpisodes;
+
+        mutable QString lastComputedEpisode;
         mutable QHash<QString, QRegularExpression> cachedRegexes;
 
         bool operator==(const AutoDownloadRuleData &other) const
@@ -135,13 +142,38 @@ namespace RSS
                     && (lastMatch == other.lastMatch)
                     && (savePath == other.savePath)
                     && (category == other.category)
-                    && (addPaused == other.addPaused);
+                    && (addPaused == other.addPaused)
+                    && (smartFilter == other.smartFilter);
         }
     };
 }
 
 using namespace RSS;
 
+QString computeEpisodeName(const QString &article)
+{
+    const QRegularExpression episodeRegex = AutoDownloader::instance()->smartEpisodeRegex();
+    const QRegularExpressionMatch match = episodeRegex.match(article);
+
+    // See if we can extract an season/episode number or date from the title
+    if (!match.hasMatch())
+        return QString();
+
+    QStringList ret;
+    for (int i = 1; i <= match.lastCapturedIndex(); ++i) {
+        QString cap = match.captured(i);
+
+        if (cap.isEmpty())
+            continue;
+
+        bool isInt = false;
+        int x = cap.toInt(&isInt);
+
+        ret.append(isInt ? QString::number(x) : cap);
+    }
+    return ret.join('x');
+}
+
 AutoDownloadRule::AutoDownloadRule(const QString &name)
     : m_dataPtr(new AutoDownloadRuleData)
 {
@@ -161,180 +193,198 @@ QRegularExpression AutoDownloadRule::cachedRegex(const QString &expression, bool
     // The cache is cleared whenever the regex/wildcard, must or must not contain fields or
     // episode filter are modified.
     Q_ASSERT(!expression.isEmpty());
-    QRegularExpression regex(m_dataPtr->cachedRegexes[expression]);
 
-    if (!regex.pattern().isEmpty())
-        return regex;
+    QRegularExpression &regex = m_dataPtr->cachedRegexes[expression];
+    if (regex.pattern().isEmpty()) {
+        regex = QRegularExpression {
+                (isRegex ? expression : Utils::String::wildcardToRegex(expression))
+                , QRegularExpression::CaseInsensitiveOption};
+    }
 
-    return m_dataPtr->cachedRegexes[expression] = QRegularExpression(isRegex ? expression : Utils::String::wildcardToRegex(expression), QRegularExpression::CaseInsensitiveOption);
+    return regex;
 }
 
-bool AutoDownloadRule::matches(const QString &articleTitle, const QString &expression) const
+bool AutoDownloadRule::matchesExpression(const QString &articleTitle, const QString &expression) const
 {
-    static QRegularExpression whitespace("\\s+");
+    const QRegularExpression whitespace {"\\s+"};
 
     if (expression.isEmpty()) {
         // A regex of the form "expr|" will always match, so do the same for wildcards
         return true;
     }
-    else if (m_dataPtr->useRegex) {
+
+    if (m_dataPtr->useRegex) {
         QRegularExpression reg(cachedRegex(expression));
         return reg.match(articleTitle).hasMatch();
     }
-    else {
-        // Only match if every wildcard token (separated by spaces) is present in the article name.
-        // Order of wildcard tokens is unimportant (if order is important, they should have used *).
-        foreach (const QString &wildcard, expression.split(whitespace, QString::SplitBehavior::SkipEmptyParts)) {
-            QRegularExpression reg(cachedRegex(wildcard, false));
 
-            if (!reg.match(articleTitle).hasMatch())
-                return false;
-        }
+    // Only match if every wildcard token (separated by spaces) is present in the article name.
+    // Order of wildcard tokens is unimportant (if order is important, they should have used *).
+    const QStringList wildcards {expression.split(whitespace, QString::SplitBehavior::SkipEmptyParts)};
+    for (const QString &wildcard : wildcards) {
+        const QRegularExpression reg {cachedRegex(wildcard, false)};
+        if (!reg.match(articleTitle).hasMatch())
+            return false;
     }
 
     return true;
 }
 
-bool AutoDownloadRule::matches(const QString &articleTitle) const
+bool AutoDownloadRule::matchesMustContainExpression(const QString &articleTitle) const
 {
-    if (!m_dataPtr->mustContain.empty()) {
-        bool logged = false;
-        bool foundMustContain = false;
+    if (m_dataPtr->mustContain.empty())
+        return true;
 
-        // Each expression is either a regex, or a set of wildcards separated by whitespace.
-        // Accept if any complete expression matches.
-        foreach (const QString &expression, m_dataPtr->mustContain) {
-            if (!logged) {
-//                qDebug() << "Checking matching" << (m_dataPtr->useRegex ? "regex:" : "wildcard expressions:") << m_dataPtr->mustContain.join("|");
-                logged = true;
-            }
+    // Each expression is either a regex, or a set of wildcards separated by whitespace.
+    // Accept if any complete expression matches.
+    for (const QString &expression : qAsConst(m_dataPtr->mustContain)) {
+        // A regex of the form "expr|" will always match, so do the same for wildcards
+        if (matchesExpression(articleTitle, expression))
+            return true;
+    }
 
-            // A regex of the form "expr|" will always match, so do the same for wildcards
-            foundMustContain = matches(articleTitle, expression);
+    return false;
+}
 
-            if (foundMustContain) {
-//                qDebug() << "Found matching" << (m_dataPtr->useRegex ? "regex:" : "wildcard expression:") << expression;
-                break;
-            }
-        }
+bool AutoDownloadRule::matchesMustNotContainExpression(const QString& articleTitle) const
+{
+    if (m_dataPtr->mustNotContain.empty())
+        return true;
 
-        if (!foundMustContain)
+    // Each expression is either a regex, or a set of wildcards separated by whitespace.
+    // Reject if any complete expression matches.
+    for (const QString &expression : qAsConst(m_dataPtr->mustNotContain)) {
+        // A regex of the form "expr|" will always match, so do the same for wildcards
+        if (matchesExpression(articleTitle, expression))
             return false;
     }
 
-    if (!m_dataPtr->mustNotContain.empty()) {
-        bool logged = false;
+    return true;
+}
 
-        // Each expression is either a regex, or a set of wildcards separated by whitespace.
-        // Reject if any complete expression matches.
-        foreach (const QString &expression, m_dataPtr->mustNotContain) {
-            if (!logged) {
-//                qDebug() << "Checking not matching" << (m_dataPtr->useRegex ? "regex:" : "wildcard expressions:") << m_dataPtr->mustNotContain.join("|");
-                logged = true;
+bool AutoDownloadRule::matchesEpisodeFilterExpression(const QString& articleTitle) const
+{
+    // Reset the lastComputedEpisode, we don't want to leak it between matches
+    m_dataPtr->lastComputedEpisode.clear();
+
+    if (m_dataPtr->episodeFilter.isEmpty())
+        return true;
+
+    const QRegularExpression filterRegex {cachedRegex("(^\\d{1,4})x(.*;$)")};
+    const QRegularExpressionMatch matcher {filterRegex.match(m_dataPtr->episodeFilter)};
+    if (!matcher.hasMatch())
+        return false;
+
+    const QString season {matcher.captured(1)};
+    const QStringList episodes {matcher.captured(2).split(';')};
+    const int seasonOurs {season.toInt()};
+
+    for (QString episode : episodes) {
+        if (episode.isEmpty())
+            continue;
+
+        // We need to trim leading zeroes, but if it's all zeros then we want episode zero.
+        while ((episode.size() > 1) && episode.startsWith('0'))
+            episode = episode.right(episode.size() - 1);
+
+        if (episode.indexOf('-') != -1) { // Range detected
+            const QString partialPattern1 {"\\bs0?(\\d{1,4})[ -_\\.]?e(0?\\d{1,4})(?:\\D|\\b)"};
+            const QString partialPattern2 {"\\b(\\d{1,4})x(0?\\d{1,4})(?:\\D|\\b)"};
+
+            // Extract partial match from article and compare as digits
+            QRegularExpressionMatch matcher = cachedRegex(partialPattern1).match(articleTitle);
+            bool matched = matcher.hasMatch();
+
+            if (!matched) {
+                matcher = cachedRegex(partialPattern2).match(articleTitle);
+                matched = matcher.hasMatch();
             }
 
-            // A regex of the form "expr|" will always match, so do the same for wildcards
-            if (matches(articleTitle, expression)) {
-//                qDebug() << "Found not matching" << (m_dataPtr->useRegex ? "regex:" : "wildcard expression:") << expression;
-                return false;
-            }
-        }
-    }
+            if (matched) {
+                const int seasonTheirs {matcher.captured(1).toInt()};
+                const int episodeTheirs {matcher.captured(2).toInt()};
 
-    if (!m_dataPtr->episodeFilter.isEmpty()) {
-//        qDebug() << "Checking episode filter:" << m_dataPtr->episodeFilter;
-        QRegularExpression f(cachedRegex("(^\\d{1,4})x(.*;$)"));
-        QRegularExpressionMatch matcher = f.match(m_dataPtr->episodeFilter);
-        bool matched = matcher.hasMatch();
-
-        if (!matched)
-            return false;
-
-        QString s = matcher.captured(1);
-        QStringList eps = matcher.captured(2).split(";");
-        int sOurs = s.toInt();
-
-        foreach (QString ep, eps) {
-            if (ep.isEmpty())
-                continue;
-
-            // We need to trim leading zeroes, but if it's all zeros then we want episode zero.
-            while (ep.size() > 1 && ep.startsWith("0"))
-                ep = ep.right(ep.size() - 1);
-
-            if (ep.indexOf('-') != -1) { // Range detected
-                QString partialPattern1 = "\\bs0?(\\d{1,4})[ -_\\.]?e(0?\\d{1,4})(?:\\D|\\b)";
-                QString partialPattern2 = "\\b(\\d{1,4})x(0?\\d{1,4})(?:\\D|\\b)";
-                QRegularExpression reg(cachedRegex(partialPattern1));
-
-                if (ep.endsWith('-')) { // Infinite range
-                    int epOurs = ep.left(ep.size() - 1).toInt();
-
-                    // Extract partial match from article and compare as digits
-                    matcher = reg.match(articleTitle);
-                    matched = matcher.hasMatch();
-
-                    if (!matched) {
-                        reg = QRegularExpression(cachedRegex(partialPattern2));
-                        matcher = reg.match(articleTitle);
-                        matched = matcher.hasMatch();
-                    }
-
-                    if (matched) {
-                        int sTheirs = matcher.captured(1).toInt();
-                        int epTheirs = matcher.captured(2).toInt();
-                        if (((sTheirs == sOurs) && (epTheirs >= epOurs)) || (sTheirs > sOurs)) {
-//                            qDebug() << "Matched episode:" << ep;
-//                            qDebug() << "Matched article:" << articleTitle;
-                            return true;
-                        }
-                    }
+                if (episode.endsWith('-')) { // Infinite range
+                    const int episodeOurs {episode.leftRef(episode.size() - 1).toInt()};
+                    if (((seasonTheirs == seasonOurs) && (episodeTheirs >= episodeOurs)) || (seasonTheirs > seasonOurs))
+                        return true;
                 }
                 else { // Normal range
-                    QStringList range = ep.split('-');
+                    const QStringList range {episode.split('-')};
                     Q_ASSERT(range.size() == 2);
                     if (range.first().toInt() > range.last().toInt())
                         continue; // Ignore this subrule completely
 
-                    int epOursFirst = range.first().toInt();
-                    int epOursLast = range.last().toInt();
-
-                    // Extract partial match from article and compare as digits
-                    matcher = reg.match(articleTitle);
-                    matched = matcher.hasMatch();
-
-                    if (!matched) {
-                        reg = QRegularExpression(cachedRegex(partialPattern2));
-                        matcher = reg.match(articleTitle);
-                        matched = matcher.hasMatch();
-                    }
-
-                    if (matched) {
-                        int sTheirs = matcher.captured(1).toInt();
-                        int epTheirs = matcher.captured(2).toInt();
-                        if ((sTheirs == sOurs) && ((epOursFirst <= epTheirs) && (epOursLast >= epTheirs))) {
-//                            qDebug() << "Matched episode:" << ep;
-//                            qDebug() << "Matched article:" << articleTitle;
-                            return true;
-                        }
-                    }
-                }
-            }
-            else { // Single number
-                QString expStr("\\b(?:s0?" + s + "[ -_\\.]?" + "e0?" + ep + "|" + s + "x" + "0?" + ep + ")(?:\\D|\\b)");
-                QRegularExpression reg(cachedRegex(expStr));
-                if (reg.match(articleTitle).hasMatch()) {
-//                    qDebug() << "Matched episode:" << ep;
-//                    qDebug() << "Matched article:" << articleTitle;
-                    return true;
+                    const int episodeOursFirst {range.first().toInt()};
+                    const int episodeOursLast {range.last().toInt()};
+                    if ((seasonTheirs == seasonOurs) && ((episodeOursFirst <= episodeTheirs) && (episodeOursLast >= episodeTheirs)))
+                        return true;
                 }
             }
         }
+        else { // Single number
+            const QString expStr {QString("\\b(?:s0?%1[ -_\\.]?e0?%2|%1x0?%2)(?:\\D|\\b)").arg(season, episode)};
+            if (cachedRegex(expStr).match(articleTitle).hasMatch())
+                return true;
+        }
+    }
 
+    return false;
+}
+
+bool AutoDownloadRule::matchesSmartEpisodeFilter(const QString& articleTitle) const
+{
+    if (!useSmartFilter())
+        return true;
+
+    const QString episodeStr = computeEpisodeName(articleTitle);
+    if (episodeStr.isEmpty())
+        return true;
+
+    // See if this episode has been downloaded before
+    const bool previouslyMatched = m_dataPtr->previouslyMatchedEpisodes.contains(episodeStr);
+    const bool isRepack = articleTitle.contains("REPACK", Qt::CaseInsensitive) || articleTitle.contains("PROPER", Qt::CaseInsensitive);
+    if (previouslyMatched && !isRepack)
+        return false;
+
+    m_dataPtr->lastComputedEpisode = episodeStr;
+    return true;
+}
+
+bool AutoDownloadRule::matches(const QVariantHash &articleData) const
+{
+    const QDateTime articleDate {articleData[Article::KeyDate].toDateTime()};
+    if (ignoreDays() > 0) {
+        if (lastMatch().isValid() && (articleDate < lastMatch().addDays(ignoreDays())))
+            return false;
+    }
+
+    const QString articleTitle {articleData[Article::KeyTitle].toString()};
+    if (!matchesMustContainExpression(articleTitle))
+        return false;
+    if (!matchesMustNotContainExpression(articleTitle))
         return false;
+    if (!matchesEpisodeFilterExpression(articleTitle))
+        return false;
+    if (!matchesSmartEpisodeFilter(articleTitle))
+        return false;
+
+    return true;
+}
+
+bool AutoDownloadRule::accepts(const QVariantHash &articleData)
+{
+    if (!matches(articleData))
+        return false;
+
+    setLastMatch(articleData[Article::KeyDate].toDateTime());
+
+    if (!m_dataPtr->lastComputedEpisode.isEmpty()) {
+        // TODO: probably need to add a marker for PROPER/REPACK to avoid duplicate downloads
+        m_dataPtr->previouslyMatchedEpisodes.append(m_dataPtr->lastComputedEpisode);
+        m_dataPtr->lastComputedEpisode.clear();
     }
 
-//    qDebug() << "Matched article:" << articleTitle;
     return true;
 }
 
@@ -367,7 +417,9 @@ QJsonObject AutoDownloadRule::toJsonObject() const
         , {Str_AssignedCategory, assignedCategory()}
         , {Str_LastMatch, lastMatch().toString(Qt::RFC2822Date)}
         , {Str_IgnoreDays, ignoreDays()}
-        , {Str_AddPaused, triStateBoolToJsonValue(addPaused())}};
+        , {Str_AddPaused, triStateBoolToJsonValue(addPaused())}
+        , {Str_SmartFilter, useSmartFilter()}
+        , {Str_PreviouslyMatched, QJsonArray::fromStringList(previouslyMatchedEpisodes())}};
 }
 
 AutoDownloadRule AutoDownloadRule::fromJsonObject(const QJsonObject &jsonObj, const QString &name)
@@ -384,6 +436,7 @@ AutoDownloadRule AutoDownloadRule::fromJsonObject(const QJsonObject &jsonObj, co
     rule.setAddPaused(jsonValueToTriStateBool(jsonObj.value(Str_AddPaused)));
     rule.setLastMatch(QDateTime::fromString(jsonObj.value(Str_LastMatch).toString(), Qt::RFC2822Date));
     rule.setIgnoreDays(jsonObj.value(Str_IgnoreDays).toInt());
+    rule.setUseSmartFilter(jsonObj.value(Str_SmartFilter).toBool(false));
 
     const QJsonValue feedsVal = jsonObj.value(Str_AffectedFeeds);
     QStringList feedURLs;
@@ -393,6 +446,17 @@ AutoDownloadRule AutoDownloadRule::fromJsonObject(const QJsonObject &jsonObj, co
         feedURLs << urlVal.toString();
     rule.setFeedURLs(feedURLs);
 
+    const QJsonValue previouslyMatchedVal = jsonObj.value(Str_PreviouslyMatched);
+    QStringList previouslyMatched;
+    if (previouslyMatchedVal.isString()) {
+        previouslyMatched << previouslyMatchedVal.toString();
+    }
+    else {
+        foreach (const QJsonValue &val, previouslyMatchedVal.toArray())
+            previouslyMatched << val.toString();
+    }
+    rule.setPreviouslyMatchedEpisodes(previouslyMatched);
+
     return rule;
 }
 
@@ -438,7 +502,7 @@ void AutoDownloadRule::setMustContain(const QString &tokens)
     if (m_dataPtr->useRegex)
         m_dataPtr->mustContain = QStringList() << tokens;
     else
-        m_dataPtr->mustContain = tokens.split("|");
+        m_dataPtr->mustContain = tokens.split('|');
 
     // Check for single empty string - if so, no condition
     if ((m_dataPtr->mustContain.size() == 1) && m_dataPtr->mustContain[0].isEmpty())
@@ -452,7 +516,7 @@ void AutoDownloadRule::setMustNotContain(const QString &tokens)
     if (m_dataPtr->useRegex)
         m_dataPtr->mustNotContain = QStringList() << tokens;
     else
-        m_dataPtr->mustNotContain = tokens.split("|");
+        m_dataPtr->mustNotContain = tokens.split('|');
 
     // Check for single empty string - if so, no condition
     if ((m_dataPtr->mustNotContain.size() == 1) && m_dataPtr->mustNotContain[0].isEmpty())
@@ -541,12 +605,22 @@ int AutoDownloadRule::ignoreDays() const
 
 QString AutoDownloadRule::mustContain() const
 {
-    return m_dataPtr->mustContain.join("|");
+    return m_dataPtr->mustContain.join('|');
 }
 
 QString AutoDownloadRule::mustNotContain() const
 {
-    return m_dataPtr->mustNotContain.join("|");
+    return m_dataPtr->mustNotContain.join('|');
+}
+
+bool AutoDownloadRule::useSmartFilter() const
+{
+    return m_dataPtr->smartFilter;
+}
+
+void AutoDownloadRule::setUseSmartFilter(bool enabled)
+{
+    m_dataPtr->smartFilter = enabled;
 }
 
 bool AutoDownloadRule::useRegex() const
@@ -560,6 +634,16 @@ void AutoDownloadRule::setUseRegex(bool enabled)
     m_dataPtr->cachedRegexes.clear();
 }
 
+QStringList AutoDownloadRule::previouslyMatchedEpisodes() const
+{
+    return m_dataPtr->previouslyMatchedEpisodes;
+}
+
+void AutoDownloadRule::setPreviouslyMatchedEpisodes(const QStringList &previouslyMatchedEpisodes)
+{
+    m_dataPtr->previouslyMatchedEpisodes = previouslyMatchedEpisodes;
+}
+
 QString AutoDownloadRule::episodeFilter() const
 {
     return m_dataPtr->episodeFilter;
