@@ -47,8 +47,8 @@
 #include "../utils/fs.h"
 #include "rss_article.h"
 #include "rss_feed.h"
-#include "rss_item.h"
 #include "rss_folder.h"
+#include "rss_item.h"
 
 const int MsecsPerMin = 60000;
 const QString ConfFolderName(QStringLiteral("rss"));
@@ -79,7 +79,7 @@ Session::Session()
     connect(m_confFileStorage, &AsyncFileStorage::failed, [](const QString &fileName, const QString &errorString)
     {
         Logger::instance()->addMessage(QString("Couldn't save RSS Session configuration in %1. Error: %2")
-                                       .arg(fileName).arg(errorString), Log::WARNING);
+                                       .arg(fileName, errorString), Log::WARNING);
     });
 
     m_dataFileStorage = new AsyncFileStorage(
@@ -89,7 +89,7 @@ Session::Session()
     connect(m_dataFileStorage, &AsyncFileStorage::failed, [](const QString &fileName, const QString &errorString)
     {
         Logger::instance()->addMessage(QString("Couldn't save RSS Session data in %1. Error: %2")
-                                       .arg(fileName).arg(errorString), Log::WARNING);
+                                       .arg(fileName, errorString), Log::WARNING);
     });
 
     m_itemsByPath.insert("", new Folder); // root folder
@@ -102,6 +102,27 @@ Session::Session()
         m_refreshTimer.start(m_refreshInterval * MsecsPerMin);
         refresh();
     }
+
+    // Remove legacy/corrupted settings
+    // (at least on Windows, QSettings is case-insensitive and it can get
+    // confused when asked about settings that differ only in their case)
+    auto settingsStorage = SettingsStorage::instance();
+    settingsStorage->removeValue("Rss/streamList");
+    settingsStorage->removeValue("Rss/streamAlias");
+    settingsStorage->removeValue("Rss/open_folders");
+    settingsStorage->removeValue("Rss/qt5/splitter_h");
+    settingsStorage->removeValue("Rss/qt5/splitterMain");
+    settingsStorage->removeValue("Rss/hosts_cookies");
+    settingsStorage->removeValue("RSS/streamList");
+    settingsStorage->removeValue("RSS/streamAlias");
+    settingsStorage->removeValue("RSS/open_folders");
+    settingsStorage->removeValue("RSS/qt5/splitter_h");
+    settingsStorage->removeValue("RSS/qt5/splitterMain");
+    settingsStorage->removeValue("RSS/hosts_cookies");
+    settingsStorage->removeValue("Rss/Session/EnableProcessing");
+    settingsStorage->removeValue("Rss/Session/RefreshInterval");
+    settingsStorage->removeValue("Rss/Session/MaxArticlesPerFeed");
+    settingsStorage->removeValue("Rss/AutoDownloader/EnableProcessing");
 }
 
 Session::~Session()
@@ -145,7 +166,7 @@ bool Session::addFeed(const QString &url, const QString &path, QString *error)
     if (!destFolder)
         return false;
 
-    addItem(new Feed(url, path, this), destFolder);
+    addItem(new Feed(generateUID(), url, path, this), destFolder);
     store();
     if (m_processingEnabled)
         feedByURL(url)->refresh();
@@ -236,7 +257,7 @@ void Session::load()
     if (!itemsFile.open(QFile::ReadOnly)) {
         Logger::instance()->addMessage(
                     QString("Couldn't read RSS Session data from %1. Error: %2")
-                    .arg(itemsFile.fileName()).arg(itemsFile.errorString()), Log::WARNING);
+                    .arg(itemsFile.fileName(), itemsFile.errorString()), Log::WARNING);
         return;
     }
 
@@ -245,7 +266,7 @@ void Session::load()
     if (jsonError.error != QJsonParseError::NoError) {
         Logger::instance()->addMessage(
                     QString("Couldn't parse RSS Session data from %1. Error: %2")
-                    .arg(itemsFile.fileName()).arg(jsonError.errorString()), Log::WARNING);
+                    .arg(itemsFile.fileName(), jsonError.errorString()), Log::WARNING);
         return;
     }
 
@@ -261,54 +282,65 @@ void Session::load()
 
 void Session::loadFolder(const QJsonObject &jsonObj, Folder *folder)
 {
+    bool updated = false;
     foreach (const QString &key, jsonObj.keys()) {
-        QJsonValue val = jsonObj[key];
+        const QJsonValue val {jsonObj[key]};
         if (val.isString()) {
+            // previous format (reduced form) doesn't contain UID
             QString url = val.toString();
             if (url.isEmpty())
                 url = key;
-            addFeedToFolder(url, key, folder);
+            addFeedToFolder(generateUID(), url, key, folder);
+            updated = true;
         }
-        else if (!val.isObject()) {
-            Logger::instance()->addMessage(
-                        QString("Couldn't load RSS Item '%1'. Invalid data format.")
-                        .arg(QString("%1\\%2").arg(folder->path()).arg(key)), Log::WARNING);
-        }
-        else {
-            QJsonObject valObj = val.toObject();
+        else if (val.isObject()) {
+            const QJsonObject valObj {val.toObject()};
             if (valObj.contains("url")) {
                 if (!valObj["url"].isString()) {
-                    Logger::instance()->addMessage(
-                                QString("Couldn't load RSS Feed '%1'. URL is required.")
-                                .arg(QString("%1\\%2").arg(folder->path()).arg(key)), Log::WARNING);
+                    LogMsg(QString("Couldn't load RSS Feed '%1'. URL is required.")
+                           .arg(QString("%1\\%2").arg(folder->path(), key)), Log::WARNING);
                     continue;
                 }
 
-                addFeedToFolder(valObj["url"].toString(), key, folder);
+                QUuid uid;
+                if (valObj.contains("uid")) {
+                    uid = QUuid {valObj["uid"].toString()};
+                    if (uid.isNull()) {
+                        LogMsg(QString("Couldn't load RSS Feed '%1'. UID is invalid.")
+                               .arg(QString("%1\\%2").arg(folder->path(), key)), Log::WARNING);
+                        continue;
+                    }
+
+                    if (m_feedsByUID.contains(uid)) {
+                        LogMsg(QString("Duplicate RSS Feed UID: %1. Configuration seems to be corrupted.")
+                               .arg(uid.toString()), Log::WARNING);
+                        continue;
+                    }
+                }
+                else {
+                    // previous format doesn't contain UID
+                    uid = generateUID();
+                    updated = true;
+                }
+
+                addFeedToFolder(uid, valObj["url"].toString(), key, folder);
             }
             else {
                 loadFolder(valObj, addSubfolder(key, folder));
             }
         }
+        else {
+            LogMsg(QString("Couldn't load RSS Item '%1'. Invalid data format.")
+                   .arg(QString("%1\\%2").arg(folder->path(), key)), Log::WARNING);
+        }
     }
+
+    if (updated)
+        store(); // convert to updated format
 }
 
 void Session::loadLegacy()
 {
-    struct LegacySettingsDeleter
-    {
-        ~LegacySettingsDeleter()
-        {
-            auto settingsStorage = SettingsStorage::instance();
-            settingsStorage->removeValue("Rss/streamList");
-            settingsStorage->removeValue("Rss/streamAlias");
-            settingsStorage->removeValue("Rss/open_folders");
-            settingsStorage->removeValue("Rss/qt5/splitter_h");
-            settingsStorage->removeValue("Rss/qt5/splitterMain");
-            settingsStorage->removeValue("Rss/hosts_cookies");
-        }
-    } legacySettingsDeleter;
-
     const QStringList legacyFeedPaths = SettingsStorage::instance()->loadValue("Rss/streamList").toStringList();
     const QStringList feedAliases = SettingsStorage::instance()->loadValue("Rss/streamAlias").toStringList();
     if (legacyFeedPaths.size() != feedAliases.size()) {
@@ -317,7 +349,7 @@ void Session::loadLegacy()
     }
 
     uint i = 0;
-    foreach (QString legacyPath, legacyFeedPaths) {
+    for (QString legacyPath : legacyFeedPaths) {
         if (Item::PathSeparator == QString(legacyPath[0]))
             legacyPath.remove(0, 1);
         const QString parentFolderPath = Item::parentPath(legacyPath);
@@ -373,9 +405,9 @@ Folder *Session::addSubfolder(const QString &name, Folder *parentFolder)
     return folder;
 }
 
-Feed *Session::addFeedToFolder(const QString &url, const QString &name, Folder *parentFolder)
+Feed *Session::addFeedToFolder(const QUuid &uid, const QString &url, const QString &name, Folder *parentFolder)
 {
-    auto feed = new Feed(url, Item::joinPath(parentFolder->path(), name), this);
+    auto feed = new Feed(uid, url, Item::joinPath(parentFolder->path(), name), this);
     addItem(feed, parentFolder);
     return feed;
 }
@@ -386,6 +418,7 @@ void Session::addItem(Item *item, Folder *destFolder)
         connect(feed, &Feed::titleChanged, this, &Session::handleFeedTitleChanged);
         connect(feed, &Feed::iconLoaded, this, &Session::feedIconLoaded);
         connect(feed, &Feed::stateChanged, this, &Session::feedStateChanged);
+        m_feedsByUID[feed->uid()] = feed;
         m_feedsByURL[feed->url()] = feed;
     }
 
@@ -466,8 +499,10 @@ void Session::handleItemAboutToBeDestroyed(Item *item)
 {
     m_itemsByPath.remove(item->path());
     auto feed = qobject_cast<Feed *>(item);
-    if (feed)
+    if (feed) {
+        m_feedsByUID.remove(feed->uid());
         m_feedsByURL.remove(feed->url());
+    }
 }
 
 void Session::handleFeedTitleChanged(Feed *feed)
@@ -478,6 +513,15 @@ void Session::handleFeedTitleChanged(Feed *feed)
         moveItem(feed, Item::joinPath(Item::parentPath(feed->path()), feed->title()));
 }
 
+QUuid Session::generateUID() const
+{
+    QUuid uid = QUuid::createUuid();
+    while (m_feedsByUID.contains(uid))
+        uid = QUuid::createUuid();
+
+    return uid;
+}
+
 int Session::maxArticlesPerFeed() const
 {
     return m_maxArticlesPerFeed;
