@@ -1,6 +1,6 @@
 /*
  * Bittorrent Client using Qt and libtorrent.
- * Copyright (C) 2012  Christophe Dumez
+ * Copyright (C) 2012  Christophe Dumez <chris@qbittorrent.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,32 +24,38 @@
  * modify file(s), you may extend this exception to your version of the file(s),
  * but you are not obligated to do so. If you do not wish to do so, delete this
  * exception statement from your version.
- *
- * Contact : chris@qbittorrent.org
  */
 
 #include "fs.h"
 
+#include <cstring>
+
+#include <QCoreApplication>
 #include <QDebug>
 #include <QDir>
+#include <QDirIterator>
 #include <QFile>
 #include <QFileInfo>
-#include <QDirIterator>
-#include <QCoreApplication>
 #include <QStorageInfo>
+#include <QRegularExpression>
+
+#include <sys/stat.h>
+#include <sys/types.h>
 
 #if defined(Q_OS_WIN)
 #include <Windows.h>
-#elif defined(Q_OS_MAC) || defined(Q_OS_FREEBSD)
+#elif defined(Q_OS_MAC) || defined(Q_OS_FREEBSD) || defined(Q_OS_OPENBSD)
 #include <sys/param.h>
 #include <sys/mount.h>
 #elif defined(Q_OS_HAIKU)
 #include <kernel/fs_info.h>
 #else
 #include <sys/vfs.h>
+#include <unistd.h>
 #endif
 
 #include "base/bittorrent/torrenthandle.h"
+#include "base/global.h"
 
 /**
  * Converts a path to a string suitable for display.
@@ -72,14 +78,14 @@ QString Utils::Fs::fromNativePath(const QString &path)
 QString Utils::Fs::fileExtension(const QString &filename)
 {
     QString ext = QString(filename).remove(QB_EXT);
-    const int pointIndex = ext.lastIndexOf(".");
+    const int pointIndex = ext.lastIndexOf('.');
     return (pointIndex >= 0) ? ext.mid(pointIndex + 1) : QString();
 }
 
 QString Utils::Fs::fileName(const QString &filePath)
 {
     QString path = fromNativePath(filePath);
-    const int slashIndex = path.lastIndexOf("/");
+    const int slashIndex = path.lastIndexOf('/');
     if (slashIndex == -1)
         return path;
     return path.mid(slashIndex + 1);
@@ -88,7 +94,7 @@ QString Utils::Fs::fileName(const QString &filePath)
 QString Utils::Fs::folderName(const QString &filePath)
 {
     QString path = fromNativePath(filePath);
-    const int slashIndex = path.lastIndexOf("/");
+    const int slashIndex = path.lastIndexOf('/');
     if (slashIndex == -1)
         return path;
     return path.left(slashIndex);
@@ -104,7 +110,7 @@ bool Utils::Fs::smartRemoveEmptyFolderTree(const QString &path)
     if (path.isEmpty() || !QDir(path).exists())
         return true;
 
-    static const QStringList deleteFilesList = {
+    const QStringList deleteFilesList = {
         // Windows
         "Thumbs.db",
         "desktop.ini",
@@ -115,15 +121,15 @@ bool Utils::Fs::smartRemoveEmptyFolderTree(const QString &path)
     };
 
     // travel from the deepest folder and remove anything unwanted on the way out.
-    QStringList dirList(path + "/");  // get all sub directories paths
+    QStringList dirList(path + '/');  // get all sub directories paths
     QDirIterator iter(path, (QDir::AllDirs | QDir::NoDotAndDotDot), QDirIterator::Subdirectories);
     while (iter.hasNext())
-        dirList << iter.next() + "/";
+        dirList << iter.next() + '/';
     // sort descending by directory depth
     std::sort(dirList.begin(), dirList.end()
-              , [](const QString &l, const QString &r) { return l.count("/") > r.count("/"); });
+              , [](const QString &l, const QString &r) { return l.count('/') > r.count('/'); });
 
-    for (const QString &p : dirList) {
+    for (const QString &p : qAsConst(dirList)) {
         // remove unwanted files
         for (const QString &f : deleteFilesList) {
             forceRemove(p + f);
@@ -133,7 +139,7 @@ bool Utils::Fs::smartRemoveEmptyFolderTree(const QString &path)
         QDir dir(p);
         QStringList tmpFileList = dir.entryList(QDir::Files);
         for (const QString &f : tmpFileList) {
-            if (f.endsWith("~"))
+            if (f.endsWith('~'))
                 forceRemove(p + f);
         }
 
@@ -145,7 +151,7 @@ bool Utils::Fs::smartRemoveEmptyFolderTree(const QString &path)
 }
 
 /**
- * Removes the file with the given file_path.
+ * Removes the file with the given filePath.
  *
  * This function will try to fix the file permissions before removing it.
  */
@@ -162,7 +168,6 @@ bool Utils::Fs::forceRemove(const QString &filePath)
 
 /**
  * Removes directory and its content recursively.
- *
  */
 void Utils::Fs::removeDirRecursive(const QString &path)
 {
@@ -205,7 +210,7 @@ bool Utils::Fs::sameFiles(const QString &path1, const QString &path2)
     if (!f2.open(QIODevice::ReadOnly)) return false;
 
     const int readSize = 1024 * 1024;  // 1 MiB
-    while(!f1.atEnd() && !f2.atEnd()) {
+    while (!f1.atEnd() && !f2.atEnd()) {
         if (f1.read(readSize) != f2.read(readSize))
             return false;
     }
@@ -214,7 +219,7 @@ bool Utils::Fs::sameFiles(const QString &path1, const QString &path2)
 
 QString Utils::Fs::toValidFileSystemName(const QString &name, bool allowSeparators, const QString &pad)
 {
-    QRegExp regex(allowSeparators ? "[:?\"*<>|]+" : "[\\\\/:?\"*<>|]+");
+    const QRegularExpression regex(allowSeparators ? "[:?\"*<>|]+" : "[\\\\/:?\"*<>|]+");
 
     QString validName = name.trimmed();
     validName.replace(regex, pad);
@@ -227,7 +232,7 @@ bool Utils::Fs::isValidFileSystemName(const QString &name, bool allowSeparators)
 {
     if (name.isEmpty()) return false;
 
-    QRegExp regex(allowSeparators ? "[:?\"*<>|]" : "[\\\\/:?\"*<>|]");
+    const QRegularExpression regex(allowSeparators ? "[:?\"*<>|]" : "[\\\\/:?\"*<>|]");
     return !name.contains(regex);
 }
 
@@ -241,9 +246,9 @@ qint64 Utils::Fs::freeDiskSpaceOnPath(const QString &path)
 QString Utils::Fs::branchPath(const QString &filePath, QString *removed)
 {
     QString ret = fromNativePath(filePath);
-    if (ret.endsWith("/"))
+    if (ret.endsWith('/'))
         ret.chop(1);
-    const int slashIndex = ret.lastIndexOf("/");
+    const int slashIndex = ret.lastIndexOf('/');
     if (slashIndex >= 0) {
         if (removed)
             *removed = ret.mid(slashIndex + 1);
@@ -281,3 +286,47 @@ QString Utils::Fs::tempPath()
     QDir().mkdir(path);
     return path;
 }
+
+bool Utils::Fs::isRegularFile(const QString &path)
+{
+    struct ::stat st;
+    if (::stat(path.toUtf8().constData(), &st) != 0) {
+        //  analyse erno and log the error
+        const auto err = errno;
+        qDebug("Could not get file stats for path '%s'. Error: %s"
+               , qUtf8Printable(path), qUtf8Printable(strerror(err)));
+        return false;
+    }
+
+    return (st.st_mode & S_IFMT) == S_IFREG;
+}
+
+#if !defined Q_OS_WIN && !defined Q_OS_HAIKU
+bool Utils::Fs::isNetworkFileSystem(const QString &path)
+{
+    QString file = path;
+    if (!file.endsWith('/'))
+        file += '/';
+    file += '.';
+
+    struct statfs buf {};
+    if (statfs(file.toLocal8Bit().constData(), &buf) != 0)
+        return false;
+
+#if defined(Q_OS_MAC) || defined(Q_OS_OPENBSD)
+    // XXX: should we make sure HAVE_STRUCT_FSSTAT_F_FSTYPENAME is defined?
+    return ((strncmp(buf.f_fstypename, "nfs", sizeof(buf.f_fstypename)) == 0)
+        || (strncmp(buf.f_fstypename, "cifs", sizeof(buf.f_fstypename)) == 0)
+        || (strncmp(buf.f_fstypename, "smbfs", sizeof(buf.f_fstypename)) == 0));
+#else
+    // usually defined in /usr/include/linux/magic.h
+    const unsigned long CIFS_MAGIC_NUMBER = 0xFF534D42;
+    const unsigned long NFS_SUPER_MAGIC = 0x6969;
+    const unsigned long SMB_SUPER_MAGIC = 0x517B;
+
+    return ((buf.f_type == CIFS_MAGIC_NUMBER)
+        || (buf.f_type == NFS_SUPER_MAGIC)
+        || (buf.f_type == SMB_SUPER_MAGIC));
+#endif
+}
+#endif
