@@ -1,6 +1,6 @@
 /*
  * Bittorrent Client using Qt and libtorrent.
- * Copyright (C) 2012  Christophe Dumez
+ * Copyright (C) 2012  Christophe Dumez <chris@qbittorrent.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,8 +24,6 @@
  * modify file(s), you may extend this exception to your version of the file(s),
  * but you are not obligated to do so. If you do not wish to do so, delete this
  * exception statement from your version.
- *
- * Contact : chris@qbittorrent.org
  */
 
 #ifndef UTILS_FS_H
@@ -52,16 +50,21 @@ namespace Utils
                 , const QString &pad = QLatin1String(" "));
         bool isValidFileSystemName(const QString &name, bool allowSeparators = false);
         qint64 freeDiskSpaceOnPath(const QString &path);
-        QString branchPath(const QString &filePath, QString *removed = 0);
+        QString branchPath(const QString &filePath, QString *removed = nullptr);
         bool sameFileNames(const QString &first, const QString &second);
         QString expandPath(const QString &path);
         QString expandPathAbs(const QString &path);
+        bool isRegularFile(const QString &path);
 
         bool smartRemoveEmptyFolderTree(const QString &path);
         bool forceRemove(const QString &filePath);
         void removeDirRecursive(const QString &path);
 
         QString tempPath();
+
+#if !defined Q_OS_WIN && !defined Q_OS_HAIKU
+        bool isNetworkFileSystem(const QString &path);
+#endif
     }
 }
 
