@@ -1,6 +1,6 @@
 /*
  * Bittorrent Client using Qt and libtorrent.
- * Copyright (C) 2006  Christophe Dumez
+ * Copyright (C) 2006  Christophe Dumez <chris@qbittorrent.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,38 +24,19 @@
  * modify file(s), you may extend this exception to your version of the file(s),
  * but you are not obligated to do so. If you do not wish to do so, delete this
  * exception statement from your version.
- *
- * Contact : chris@qbittorrent.org
  */
 
-#include <QUrl>
-#include <QDir>
-#include <QFileInfo>
-#include <QDateTime>
-#include <QByteArray>
-#include <QDebug>
-#include <QProcess>
-#include <QRegularExpression>
-#include <QSysInfo>
+#include "misc.h"
+
 #include <boost/version.hpp>
 #include <libtorrent/version.hpp>
 
-#ifdef DISABLE_GUI
-#include <QCoreApplication>
-#else
-#include <QApplication>
-#include <QDesktopWidget>
-#include <QStyle>
-#endif
-
 #ifdef Q_OS_WIN
 #include <windows.h>
-#include <powrprof.h>
 #include <Shlobj.h>
-const int UNLEN = 256;
 #else
-#include <unistd.h>
 #include <sys/types.h>
+#include <unistd.h>
 #endif
 
 #ifdef Q_OS_MAC
@@ -63,33 +44,47 @@ const int UNLEN = 256;
 #include <Carbon/Carbon.h>
 #endif
 
-#ifndef DISABLE_GUI
+#include <QByteArray>
+#include <QDebug>
+#include <QFileInfo>
+#include <QProcess>
+#include <QRegularExpression>
+#include <QSysInfo>
+#include <QUrl>
+
+#ifdef DISABLE_GUI
+#include <QCoreApplication>
+#else
+#include <QApplication>
+#include <QDesktopServices>
+#include <QDesktopWidget>
+#include <QStyle>
 #if (defined(Q_OS_UNIX) && !defined(Q_OS_MAC)) && defined(QT_DBUS_LIB)
 #include <QDBusInterface>
 #include <QDBusMessage>
 #endif
-#endif // DISABLE_GUI
-
-#ifndef DISABLE_GUI
-#include <QDesktopServices>
-#include <QProcess>
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
+#include "base/utils/version.h"
+#endif
 #endif
 
-#include "base/utils/string.h"
-#include "base/unicodestrings.h"
 #include "base/logger.h"
-#include "misc.h"
+#include "base/unicodestrings.h"
+#include "base/utils/string.h"
 #include "fs.h"
 
-static struct { const char *source; const char *comment; } units[] = {
-    QT_TRANSLATE_NOOP3("misc", "B", "bytes"),
-    QT_TRANSLATE_NOOP3("misc", "KiB", "kibibytes (1024 bytes)"),
-    QT_TRANSLATE_NOOP3("misc", "MiB", "mebibytes (1024 kibibytes)"),
-    QT_TRANSLATE_NOOP3("misc", "GiB", "gibibytes (1024 mibibytes)"),
-    QT_TRANSLATE_NOOP3("misc", "TiB", "tebibytes (1024 gibibytes)"),
-    QT_TRANSLATE_NOOP3("misc", "PiB", "pebibytes (1024 tebibytes)"),
-    QT_TRANSLATE_NOOP3("misc", "EiB", "exbibytes (1024 pebibytes)")
-};
+namespace
+{
+    const struct { const char *source; const char *comment; } units[] = {
+        QT_TRANSLATE_NOOP3("misc", "B", "bytes"),
+        QT_TRANSLATE_NOOP3("misc", "KiB", "kibibytes (1024 bytes)"),
+        QT_TRANSLATE_NOOP3("misc", "MiB", "mebibytes (1024 kibibytes)"),
+        QT_TRANSLATE_NOOP3("misc", "GiB", "gibibytes (1024 mibibytes)"),
+        QT_TRANSLATE_NOOP3("misc", "TiB", "tebibytes (1024 gibibytes)"),
+        QT_TRANSLATE_NOOP3("misc", "PiB", "pebibytes (1024 tebibytes)"),
+        QT_TRANSLATE_NOOP3("misc", "EiB", "exbibytes (1024 pebibytes)")
+    };
+}
 
 void Utils::Misc::shutdownComputer(const ShutdownDialogAction &action)
 {
@@ -115,12 +110,23 @@ void Utils::Misc::shutdownComputer(const ShutdownDialogAction &action)
     if (GetLastError() != ERROR_SUCCESS)
         return;
 
-    if (action == ShutdownDialogAction::Suspend)
-        SetSuspendState(false, false, false);
-    else if (action == ShutdownDialogAction::Hibernate)
-        SetSuspendState(true, false, false);
-    else
-        InitiateSystemShutdownA(0, QCoreApplication::translate("misc", "qBittorrent will shutdown the computer now because all downloads are complete.").toLocal8Bit().data(), 10, true, false);
+    using PSETSUSPENDSTATE = BOOLEAN (WINAPI *)(BOOLEAN, BOOLEAN, BOOLEAN);
+    const auto setSuspendState = Utils::Misc::loadWinAPI<PSETSUSPENDSTATE>("PowrProf.dll", "SetSuspendState");
+
+    if (action == ShutdownDialogAction::Suspend) {
+        if (setSuspendState)
+            setSuspendState(false, false, false);
+    }
+    else if (action == ShutdownDialogAction::Hibernate) {
+        if (setSuspendState)
+            setSuspendState(true, false, false);
+    }
+    else {
+        const QString msg = QCoreApplication::translate("misc", "qBittorrent will shutdown the computer now because all downloads are complete.");
+        std::unique_ptr<wchar_t[]> msgWchar(new wchar_t[msg.length() + 1] {});
+        msg.toWCharArray(msgWchar.get());
+        ::InitiateSystemShutdownW(nullptr, msgWchar.get(), 10, true, false);
+    }
 
     // Disable shutdown privilege.
     tkp.Privileges[0].Attributes = 0;
@@ -133,7 +139,7 @@ void Utils::Misc::shutdownComputer(const ShutdownDialogAction &action)
     else
         EventToSend = kAEShutDown;
     AEAddressDesc targetDesc;
-    static const ProcessSerialNumber kPSNOfSystemProcess = { 0, kSystemProcess };
+    const ProcessSerialNumber kPSNOfSystemProcess = {0, kSystemProcess};
     AppleEvent eventReply = {typeNull, NULL};
     AppleEvent appleEventToSend = {typeNull, NULL};
 
@@ -229,106 +235,8 @@ QPoint Utils::Misc::screenCenter(const QWidget *w)
     QRect r = desktop->availableGeometry(scrn);
     return QPoint(r.x() + (r.width() - w->frameSize().width()) / 2, r.y() + (r.height() - w->frameSize().height()) / 2);
 }
-
 #endif
 
-/**
- * Detects the python version.
- */
-int Utils::Misc::pythonVersion()
-{
-    static int version = -1;
-    if (version < 0) {
-        QString versionComplete = pythonVersionComplete().trimmed();
-        QStringList splitted = versionComplete.split('.');
-        if (splitted.size() > 1) {
-            int highVer = splitted.at(0).toInt();
-            if ((highVer == 2) || (highVer == 3))
-                version = highVer;
-        }
-    }
-    return version;
-}
-
-/**
- * Detects the python executable by calling "python --version".
- */
-QString Utils::Misc::pythonExecutable()
-{
-    static QString executable;
-    if (executable.isEmpty()) {
-        QProcess pythonProc;
-#if defined(Q_OS_UNIX)
-        /*
-         * On Unix-Like Systems python2 and python3 should always exist
-         * http://legacy.python.org/dev/peps/pep-0394/
-         */
-        pythonProc.start("python3", QStringList() << "--version", QIODevice::ReadOnly);
-        if (pythonProc.waitForFinished() && (pythonProc.exitCode() == 0)) {
-            executable = "python3";
-            return executable;
-        }
-        pythonProc.start("python2", QStringList() << "--version", QIODevice::ReadOnly);
-        if (pythonProc.waitForFinished() && (pythonProc.exitCode() == 0)) {
-            executable = "python2";
-            return executable;
-        }
-#endif
-        // Look for "python" in Windows and in UNIX if "python2" and "python3" are
-        // not detected.
-        pythonProc.start("python", QStringList() << "--version", QIODevice::ReadOnly);
-        if (pythonProc.waitForFinished() && (pythonProc.exitCode() == 0))
-            executable = "python";
-        else
-            Logger::instance()->addMessage(QCoreApplication::translate("misc", "Python not detected"), Log::INFO);
-    }
-    return executable;
-}
-
-/**
- * Returns the complete python version
- * eg 2.7.9
- * Make sure to have setup python first
- */
-QString Utils::Misc::pythonVersionComplete()
-{
-    static QString version;
-    if (version.isEmpty()) {
-        if (pythonExecutable().isEmpty())
-            return version;
-        QProcess pythonProc;
-        pythonProc.start(pythonExecutable(), QStringList() << "--version", QIODevice::ReadOnly);
-        if (pythonProc.waitForFinished() && (pythonProc.exitCode() == 0)) {
-            QByteArray output = pythonProc.readAllStandardOutput();
-            if (output.isEmpty())
-                output = pythonProc.readAllStandardError();
-
-            // Software 'Anaconda' installs its own python interpreter
-            // and `python --version` returns a string like this:
-            // `Python 3.4.3 :: Anaconda 2.3.0 (64-bit)`
-            const QList<QByteArray> outSplit = output.split(' ');
-            if (outSplit.size() > 1) {
-                version = outSplit.at(1).trimmed();
-                Logger::instance()->addMessage(QCoreApplication::translate("misc", "Python version: %1").arg(version), Log::INFO);
-            }
-
-            // If python doesn't report a 3-piece version e.g. 3.6.1
-            // then fill the missing pieces with zero
-            const QStringList verSplit = version.split('.', QString::SkipEmptyParts);
-            if (verSplit.size() < 3) {
-                for (int i = verSplit.size(); i < 3; ++i) {
-                    if (version.endsWith('.'))
-                        version.append('0');
-                    else
-                        version.append(".0");
-                }
-                Logger::instance()->addMessage(QCoreApplication::translate("misc", "Normalized Python version: %1").arg(version), Log::INFO);
-            }
-        }
-    }
-    return version;
-}
-
 QString Utils::Misc::unitString(Utils::Misc::SizeUnit unit)
 {
     return QCoreApplication::translate("misc",
@@ -389,55 +297,50 @@ qlonglong Utils::Misc::sizeInBytes(qreal size, Utils::Misc::SizeUnit unit)
 
 bool Utils::Misc::isPreviewable(const QString &extension)
 {
-    static QSet<QString> multimedia_extensions;
-    if (multimedia_extensions.empty()) {
-        multimedia_extensions.insert("3GP");
-        multimedia_extensions.insert("AAC");
-        multimedia_extensions.insert("AC3");
-        multimedia_extensions.insert("AIF");
-        multimedia_extensions.insert("AIFC");
-        multimedia_extensions.insert("AIFF");
-        multimedia_extensions.insert("ASF");
-        multimedia_extensions.insert("AU");
-        multimedia_extensions.insert("AVI");
-        multimedia_extensions.insert("FLAC");
-        multimedia_extensions.insert("FLV");
-        multimedia_extensions.insert("M3U");
-        multimedia_extensions.insert("M4A");
-        multimedia_extensions.insert("M4P");
-        multimedia_extensions.insert("M4V");
-        multimedia_extensions.insert("MID");
-        multimedia_extensions.insert("MKV");
-        multimedia_extensions.insert("MOV");
-        multimedia_extensions.insert("MP2");
-        multimedia_extensions.insert("MP3");
-        multimedia_extensions.insert("MP4");
-        multimedia_extensions.insert("MPC");
-        multimedia_extensions.insert("MPE");
-        multimedia_extensions.insert("MPEG");
-        multimedia_extensions.insert("MPG");
-        multimedia_extensions.insert("MPP");
-        multimedia_extensions.insert("OGG");
-        multimedia_extensions.insert("OGM");
-        multimedia_extensions.insert("OGV");
-        multimedia_extensions.insert("QT");
-        multimedia_extensions.insert("RA");
-        multimedia_extensions.insert("RAM");
-        multimedia_extensions.insert("RM");
-        multimedia_extensions.insert("RMV");
-        multimedia_extensions.insert("RMVB");
-        multimedia_extensions.insert("SWA");
-        multimedia_extensions.insert("SWF");
-        multimedia_extensions.insert("VOB");
-        multimedia_extensions.insert("WAV");
-        multimedia_extensions.insert("WMA");
-        multimedia_extensions.insert("WMV");
-    }
-
-    if (extension.isEmpty())
-        return false;
-
-    return multimedia_extensions.contains(extension.toUpper());
+    static const QSet<QString> multimediaExtensions = {
+        "3GP",
+        "AAC",
+        "AC3",
+        "AIF",
+        "AIFC",
+        "AIFF",
+        "ASF",
+        "AU",
+        "AVI",
+        "FLAC",
+        "FLV",
+        "M3U",
+        "M4A",
+        "M4P",
+        "M4V",
+        "MID",
+        "MKV",
+        "MOV",
+        "MP2",
+        "MP3",
+        "MP4",
+        "MPC",
+        "MPE",
+        "MPEG",
+        "MPG",
+        "MPP",
+        "OGG",
+        "OGM",
+        "OGV",
+        "QT",
+        "RA",
+        "RAM",
+        "RM",
+        "RMV",
+        "RMVB",
+        "SWA",
+        "SWF",
+        "VOB",
+        "WAV",
+        "WMA",
+        "WMV"
+    };
+    return multimediaExtensions.contains(extension.toUpper());
 }
 
 // Take a number of seconds and return an user-friendly
@@ -460,12 +363,12 @@ QString Utils::Misc::userFriendlyDuration(qlonglong seconds)
     qlonglong hours = minutes / 60;
     minutes -= hours * 60;
     if (hours < 24)
-        return QCoreApplication::translate("misc", "%1h %2m", "e.g: 3hours 5minutes").arg(QString::number(hours)).arg(QString::number(minutes));
+        return QCoreApplication::translate("misc", "%1h %2m", "e.g: 3hours 5minutes").arg(QString::number(hours), QString::number(minutes));
 
     qlonglong days = hours / 24;
     hours -= days * 24;
     if (days < 100)
-        return QCoreApplication::translate("misc", "%1d %2h", "e.g: 2days 10hours").arg(QString::number(days)).arg(QString::number(hours));
+        return QCoreApplication::translate("misc", "%1d %2h", "e.g: 2days 10hours").arg(QString::number(days), QString::number(hours));
 
     return QString::fromUtf8(C_INFINITY);
 }
@@ -474,6 +377,7 @@ QString Utils::Misc::getUserIDString()
 {
     QString uid = "0";
 #ifdef Q_OS_WIN
+    const int UNLEN = 256;
     WCHAR buffer[UNLEN + 1] = {0};
     DWORD buffer_len = sizeof(buffer) / sizeof(*buffer);
     if (GetUserNameW(buffer, &buffer_len))
@@ -516,10 +420,10 @@ bool Utils::Misc::isUrl(const QString &s)
     return reURLScheme.match(QUrl(s).scheme()).hasMatch();
 }
 
-QString Utils::Misc::parseHtmlLinks(const QString &raw_text)
+QString Utils::Misc::parseHtmlLinks(const QString &rawText)
 {
-    QString result = raw_text;
-    static QRegExp reURL(
+    QString result = rawText;
+    static const QRegularExpression reURL(
         "(\\s|^)"                                             // start with whitespace or beginning of line
         "("
         "("                                              // case 1 -- URL with scheme
@@ -566,12 +470,11 @@ QString Utils::Misc::parseHtmlLinks(const QString &raw_text)
         ")"
         );
 
-
     // Capture links
     result.replace(reURL, "\\1<a href=\"\\2\">\\2</a>");
 
     // Capture links without scheme
-    static QRegExp reNoScheme("<a\\s+href=\"(?!http(s?))([a-zA-Z0-9\\?%=&/_\\.-:#]+)\\s*\">");
+    static const QRegularExpression reNoScheme("<a\\s+href=\"(?!https?)([a-zA-Z0-9\\?%=&/_\\.-:#]+)\\s*\">");
     result.replace(reNoScheme, "<a href=\"http://\\1\">");
 
     // to preserve plain text formatting
@@ -597,8 +500,8 @@ void Utils::Misc::openFolderSelect(const QString &absolutePath)
 {
     const QString path = Utils::Fs::fromNativePath(absolutePath);
     // If the item to select doesn't exist, try to open its parent
-    if (!QFileInfo(path).exists()) {
-        openPath(path.left(path.lastIndexOf("/")));
+    if (!QFileInfo::exists(path)) {
+        openPath(path.left(path.lastIndexOf('/')));
         return;
     }
 #ifdef Q_OS_WIN
@@ -612,39 +515,38 @@ void Utils::Misc::openFolderSelect(const QString &absolutePath)
         ::CoUninitialize();
 #elif defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
     QProcess proc;
-    proc.start("xdg-mime", QStringList() << "query" << "default" << "inode/directory");
+    proc.start("xdg-mime", {"query", "default", "inode/directory"});
     proc.waitForFinished();
     QString output = proc.readLine().simplified();
-    if ((output == "dolphin.desktop") || (output == "org.kde.dolphin.desktop"))
-        proc.startDetached("dolphin", QStringList() << "--select" << Utils::Fs::toNativePath(path));
+    if ((output == "dolphin.desktop") || (output == "org.kde.dolphin.desktop")) {
+        proc.startDetached("dolphin", {"--select", Utils::Fs::toNativePath(path)});
+    }
     else if ((output == "nautilus.desktop") || (output == "org.gnome.Nautilus.desktop")
-             || (output == "nautilus-folder-handler.desktop"))
-        proc.startDetached("nautilus", QStringList() << "--no-desktop" << Utils::Fs::toNativePath(path));
-    else if (output == "nemo.desktop")
-        proc.startDetached("nemo", QStringList() << "--no-desktop" << Utils::Fs::toNativePath(path));
-    else if ((output == "konqueror.desktop") || (output == "kfmclient_dir.desktop"))
-        proc.startDetached("konqueror", QStringList() << "--select" << Utils::Fs::toNativePath(path));
-    else
+                 || (output == "nautilus-folder-handler.desktop")) {
+        proc.start("nautilus", {"--version"});
+        proc.waitForFinished();
+        const QString nautilusVerStr = QString(proc.readLine()).remove(QRegularExpression("[^0-9.]"));
+        using NautilusVersion = Utils::Version<int, 3>;
+        if (NautilusVersion::tryParse(nautilusVerStr, {1, 0, 0}) > NautilusVersion {3, 28})
+            proc.startDetached("nautilus", {Utils::Fs::toNativePath(path)});
+        else
+            proc.startDetached("nautilus", {"--no-desktop", Utils::Fs::toNativePath(path)});
+    }
+    else if (output == "nemo.desktop") {
+        proc.startDetached("nemo", {"--no-desktop", Utils::Fs::toNativePath(path)});
+    }
+    else if ((output == "konqueror.desktop") || (output == "kfmclient_dir.desktop")) {
+        proc.startDetached("konqueror", {"--select", Utils::Fs::toNativePath(path)});
+    }
+    else {
         // "caja" manager can't pinpoint the file, see: https://github.com/qbittorrent/qBittorrent/issues/5003
-        openPath(path.left(path.lastIndexOf("/")));
+        openPath(path.left(path.lastIndexOf('/')));
+    }
 #else
-    openPath(path.left(path.lastIndexOf("/")));
+    openPath(path.left(path.lastIndexOf('/')));
 #endif
 }
 
-QSize Utils::Misc::smallIconSize()
-{
-    // Get DPI scaled icon size (device-dependent), see QT source
-    int s = QApplication::style()->pixelMetric(QStyle::PM_SmallIconSize);
-    return QSize(s, s);
-}
-
-QSize Utils::Misc::largeIconSize()
-{
-    // Get DPI scaled icon size (device-dependent), see QT source
-    int s = QApplication::style()->pixelMetric(QStyle::PM_LargeIconSize);
-    return QSize(s, s);
-}
 #endif // DISABLE_GUI
 
 QString Utils::Misc::osName()
@@ -652,9 +554,9 @@ QString Utils::Misc::osName()
     // static initialization for usage in signal handler
     static const QString name =
         QString("%1 %2 %3")
-        .arg(QSysInfo::prettyProductName())
-        .arg(QSysInfo::kernelVersion())
-        .arg(QSysInfo::currentCpuArchitecture());
+        .arg(QSysInfo::prettyProductName()
+            , QSysInfo::kernelVersion()
+            , QSysInfo::currentCpuArchitecture());
     return name;
 }
 
