@@ -35,8 +35,9 @@
 #include "cookiesmodel.h"
 #include "guiiconprovider.h"
 #include "ui_cookiesdialog.h"
+#include "utils.h"
 
-#define SETTINGS_KEY(name) "CookiesDialog/" name
+#define SETTINGS_KEY(name) QStringLiteral("CookiesDialog/" name)
 const QString KEY_SIZE = SETTINGS_KEY("Size");
 const QString KEY_COOKIESVIEWSTATE = SETTINGS_KEY("CookiesViewState");
 
@@ -50,6 +51,8 @@ CookiesDialog::CookiesDialog(QWidget *parent)
     setWindowIcon(GuiIconProvider::instance()->getIcon("preferences-web-browser-cookies"));
     m_ui->buttonAdd->setIcon(GuiIconProvider::instance()->getIcon("list-add"));
     m_ui->buttonDelete->setIcon(GuiIconProvider::instance()->getIcon("list-remove"));
+    m_ui->buttonAdd->setIconSize(Utils::Gui::mediumIconSize());
+    m_ui->buttonDelete->setIconSize(Utils::Gui::mediumIconSize());
 
     m_ui->treeView->setModel(m_cookiesModel);
     if (m_cookiesModel->rowCount() > 0)
@@ -57,7 +60,7 @@ CookiesDialog::CookiesDialog(QWidget *parent)
                     m_cookiesModel->index(0, 0),
                     QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
 
-    resize(SettingsStorage::instance()->loadValue(KEY_SIZE, size()).toSize());
+    Utils::Gui::resize(this, SettingsStorage::instance()->loadValue(KEY_SIZE).toSize());
     m_ui->treeView->header()->restoreState(
                 SettingsStorage::instance()->loadValue(KEY_COOKIESVIEWSTATE).toByteArray());
 }
