@@ -1,6 +1,6 @@
 /*
- * Bittorrent Client using Qt4 and libtorrent.
- * Copyright (C) 2010  Christophe Dumez
+ * Bittorrent Client using Qt and libtorrent.
+ * Copyright (C) 2010  Christophe Dumez <chris@qbittorrent.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,32 +24,24 @@
  * modify file(s), you may extend this exception to your version of the file(s),
  * but you are not obligated to do so. If you do not wish to do so, delete this
  * exception statement from your version.
- *
- * Contact : chris@qbittorrent.org
  */
 
-#include <QXmlStreamReader>
-#include <QDesktopServices>
-#include <QDebug>
-#include <QRegExp>
-#include <QStringList>
-
-#include "base/utils/fs.h"
-#include "base/net/downloadmanager.h"
-#include "base/net/downloadhandler.h"
 #include "programupdater.h"
 
+#include <QDebug>
+#include <QDesktopServices>
+#include <QRegularExpression>
+#include <QStringList>
+#include <QSysInfo>
+#include <QXmlStreamReader>
+
+#include "base/net/downloadhandler.h"
+#include "base/net/downloadmanager.h"
+#include "base/utils/fs.h"
+
 namespace
 {
-    const QString RSS_URL("https://www.fosshub.com/software/feedqBittorent");
-
-#ifdef Q_OS_MAC
-    const QString OS_TYPE("Mac OS X");
-#elif defined(Q_OS_WIN) && (defined(__x86_64__) || defined(_M_X64))
-    const QString OS_TYPE("Windows x64");
-#else
-    const QString OS_TYPE("Windows");
-#endif
+    const QString RSS_URL {QStringLiteral("https://www.fosshub.com/software/feedqBittorent")};
 
     QString getStringValue(QXmlStreamReader &xml);
 }
@@ -62,22 +54,29 @@ ProgramUpdater::ProgramUpdater(QObject *parent, bool invokedByUser)
 
 void ProgramUpdater::checkForUpdates()
 {
-    Net::DownloadHandler *handler = Net::DownloadManager::instance()->downloadUrl(
-                RSS_URL, false, 0, false,
-                // Don't change this User-Agent. In case our updater goes haywire,
-                // the filehost can identify it and contact us.
-                "qBittorrent/" QBT_VERSION_2 " ProgramUpdater (www.qbittorrent.org)");
-    connect(handler, SIGNAL(downloadFinished(QString,QByteArray)), SLOT(rssDownloadFinished(QString,QByteArray)));
-    connect(handler, SIGNAL(downloadFailed(QString,QString)), SLOT(rssDownloadFailed(QString,QString)));
+    // Don't change this User-Agent. In case our updater goes haywire,
+    // the filehost can identify it and contact us.
+    Net::DownloadHandler *handler = Net::DownloadManager::instance()->download(
+                Net::DownloadRequest(RSS_URL).userAgent("qBittorrent/" QBT_VERSION_2 " ProgramUpdater (www.qbittorrent.org)"));
+    connect(handler, static_cast<void (Net::DownloadHandler::*)(const QString &, const QByteArray &)>(&Net::DownloadHandler::downloadFinished)
+            , this, &ProgramUpdater::rssDownloadFinished);
+    connect(handler, &Net::DownloadHandler::downloadFailed, this, &ProgramUpdater::rssDownloadFailed);
 }
 
 void ProgramUpdater::rssDownloadFinished(const QString &url, const QByteArray &data)
 {
     Q_UNUSED(url);
-
     qDebug("Finished downloading the new qBittorrent updates RSS");
-    QString version;
 
+#ifdef Q_OS_MAC
+    const QString OS_TYPE {"Mac OS X"};
+#elif defined(Q_OS_WIN)
+    const QString OS_TYPE {((QSysInfo::windowsVersion() >= QSysInfo::WV_WINDOWS7)
+            && QSysInfo::currentCpuArchitecture().endsWith("64"))
+        ? "Windows x64" : "Windows"};
+#endif
+
+    QString version;
     QXmlStreamReader xml(data);
     bool inItem = false;
     QString updateLink;
@@ -136,9 +135,9 @@ void ProgramUpdater::updateProgram()
 
 bool ProgramUpdater::isVersionMoreRecent(const QString &remoteVersion) const
 {
-    QRegExp regVer("([0-9.]+)");
-    if (regVer.indexIn(QBT_VERSION) >= 0) {
-        QString localVersion = regVer.cap(1);
+    const QRegularExpressionMatch regVerMatch = QRegularExpression("([0-9.]+)").match(QBT_VERSION);
+    if (regVerMatch.hasMatch()) {
+        QString localVersion = regVerMatch.captured(1);
         qDebug() << Q_FUNC_INFO << "local version:" << localVersion << "/" << QBT_VERSION;
         QStringList remoteParts = remoteVersion.split('.');
         QStringList localParts = localVersion.split('.');
@@ -152,8 +151,8 @@ bool ProgramUpdater::isVersionMoreRecent(const QString &remoteVersion) const
         if (remoteParts.size() > localParts.size())
             return true;
         // versions are equal, check if the local version is a development release, in which case it is older (2.9.2beta < 2.9.2)
-        QRegExp regDevel("(alpha|beta|rc)");
-        if (regDevel.indexIn(QBT_VERSION) >= 0)
+        const QRegularExpressionMatch regDevelMatch = QRegularExpression("(alpha|beta|rc)").match(QBT_VERSION);
+        if (regDevelMatch.hasMatch())
             return true;
     }
     return false;
