@@ -30,7 +30,6 @@
 #define PEERLISTSORTMODEL_H
 
 #include <QSortFilterProxyModel>
-#include <QStringList>
 
 #include "peerlistdelegate.h"
 
@@ -53,17 +52,11 @@ protected:
                 const QString strL = left.data().toString();
                 const QString strR = right.data().toString();
                 const int result = Utils::String::naturalCompare(strL, strR, Qt::CaseInsensitive);
-                if (result != 0)
-                    return (result < 0);
-
-                return (left < right);
+                return (result < 0);
             }
             break;
         default:
-            if (left.data() != right.data())
-                return QSortFilterProxyModel::lessThan(left, right);
-
-            return (left < right);
+            return QSortFilterProxyModel::lessThan(left, right);
         };
     }
 };
