@@ -35,6 +35,7 @@
 #include "base/utils/misc.h"
 #include "base/utils/string.h"
 #include "ui_statsdialog.h"
+#include "utils.h"
 
 StatsDialog::StatsDialog(QWidget *parent)
     : QDialog(parent)
@@ -48,6 +49,7 @@ StatsDialog::StatsDialog(QWidget *parent)
     connect(BitTorrent::Session::instance(), &BitTorrent::Session::statsUpdated
             , this, &StatsDialog::update);
 
+    Utils::Gui::resize(this);
     show();
 }
 
@@ -61,7 +63,7 @@ void StatsDialog::update()
     const BitTorrent::SessionStatus &ss = BitTorrent::Session::instance()->status();
     const BitTorrent::CacheStatus &cs = BitTorrent::Session::instance()->cacheStatus();
 
-    // Alltime DL/UL
+    // All-time DL/UL
     quint64 atd = BitTorrent::Session::instance()->getAlltimeDL();
     quint64 atu = BitTorrent::Session::instance()->getAlltimeUL();
     m_ui->labelAlltimeDL->setText(Utils::Misc::friendlyUnit(atd));
@@ -75,7 +77,10 @@ void StatsDialog::update()
                 : "-");
     // Cache hits
     qreal readRatio = cs.readRatio;
-    m_ui->labelCacheHits->setText((readRatio >= 0) ? Utils::String::fromDouble(100 * readRatio, 2) : "-");
+    m_ui->labelCacheHits->setText(QString("%1%").arg(
+        readRatio > 0
+        ? Utils::String::fromDouble(100 * readRatio, 2)
+        : "0"));
     // Buffers size
     m_ui->labelTotalBuf->setText(Utils::Misc::friendlyUnit(cs.totalUsedBuffers * 16 * 1024));
     // Disk overload (100%) equivalent
