@@ -40,7 +40,8 @@
 #include "base/bittorrent/sessionstatus.h"
 #include "base/utils/misc.h"
 #include "guiiconprovider.h"
-#include "speedlimitdlg.h"
+#include "speedlimitdialog.h"
+#include "utils.h"
 
 StatusBar::StatusBar(QWidget *parent)
     : QStatusBar(parent)
@@ -62,15 +63,14 @@ StatusBar::StatusBar(QWidget *parent)
     m_connecStatusLblIcon->setFlat(true);
     m_connecStatusLblIcon->setFocusPolicy(Qt::NoFocus);
     m_connecStatusLblIcon->setCursor(Qt::PointingHandCursor);
-    m_connecStatusLblIcon->setIcon(QIcon(":/icons/skin/firewalled.png"));
+    m_connecStatusLblIcon->setIcon(QIcon(":/icons/skin/firewalled.svg"));
     m_connecStatusLblIcon->setToolTip(
-                QString(QLatin1String("<b>%1</b><br><i>%2</i>"))
-                .arg(tr("Connection status:"))
-                .arg(tr("No direct connections. This may indicate network configuration problems.")));
+        QString(QLatin1String("<b>%1</b><br><i>%2</i>")).arg(tr("Connection status:")
+            , tr("No direct connections. This may indicate network configuration problems.")));
     connect(m_connecStatusLblIcon, &QAbstractButton::clicked, this, &StatusBar::connectionButtonClicked);
 
     m_dlSpeedLbl = new QPushButton(this);
-    m_dlSpeedLbl->setIcon(QIcon(":/icons/skin/download.png"));
+    m_dlSpeedLbl->setIcon(QIcon(":/icons/skin/download.svg"));
     connect(m_dlSpeedLbl, &QAbstractButton::clicked, this, &StatusBar::capDownloadSpeed);
     m_dlSpeedLbl->setFlat(true);
     m_dlSpeedLbl->setFocusPolicy(Qt::NoFocus);
@@ -79,7 +79,7 @@ StatusBar::StatusBar(QWidget *parent)
     m_dlSpeedLbl->setMinimumWidth(200);
 
     m_upSpeedLbl = new QPushButton(this);
-    m_upSpeedLbl->setIcon(QIcon(":/icons/skin/seeding.png"));
+    m_upSpeedLbl->setIcon(QIcon(":/icons/skin/seeding.svg"));
     connect(m_upSpeedLbl, &QAbstractButton::clicked, this, &StatusBar::capUploadSpeed);
     m_upSpeedLbl->setFlat(true);
     m_upSpeedLbl->setFocusPolicy(Qt::NoFocus);
@@ -100,15 +100,15 @@ StatusBar::StatusBar(QWidget *parent)
     // Because on some platforms the default icon size is bigger
     // and it will result in taller/fatter statusbar, even if the
     // icons are actually 16x16
-    m_connecStatusLblIcon->setIconSize(QSize(16, 16));
-    m_dlSpeedLbl->setIconSize(QSize(16, 16));
-    m_upSpeedLbl->setIconSize(QSize(16, 16));
-    m_altSpeedsBtn->setIconSize(QSize(28, 16));
+    m_connecStatusLblIcon->setIconSize(Utils::Gui::smallIconSize());
+    m_dlSpeedLbl->setIconSize(Utils::Gui::smallIconSize());
+    m_upSpeedLbl->setIconSize(Utils::Gui::smallIconSize());
+    m_altSpeedsBtn->setIconSize(QSize(Utils::Gui::mediumIconSize().width(), Utils::Gui::smallIconSize().height()));
 
     // Set to the known maximum width(plus some padding)
     // so the speed widgets will take the rest of the space
-    m_connecStatusLblIcon->setMaximumWidth(16 + 6);
-    m_altSpeedsBtn->setMaximumWidth(28 + 6);
+    m_connecStatusLblIcon->setMaximumWidth(Utils::Gui::largeIconSize().width());
+    m_altSpeedsBtn->setMaximumWidth(Utils::Gui::largeIconSize().width());
 
     QFrame *statusSep1 = new QFrame(this);
     statusSep1->setFrameStyle(QFrame::VLine);
@@ -174,17 +174,17 @@ void StatusBar::updateConnectionStatus()
     const BitTorrent::SessionStatus &sessionStatus = BitTorrent::Session::instance()->status();
 
     if (!BitTorrent::Session::instance()->isListening()) {
-        m_connecStatusLblIcon->setIcon(QIcon(QLatin1String(":/icons/skin/disconnected.png")));
+        m_connecStatusLblIcon->setIcon(QIcon(QLatin1String(":/icons/skin/disconnected.svg")));
         m_connecStatusLblIcon->setToolTip(QLatin1String("<b>") + tr("Connection Status:") + QLatin1String("</b><br>") + tr("Offline. This usually means that qBittorrent failed to listen on the selected port for incoming connections."));
     }
     else {
         if (sessionStatus.hasIncomingConnections) {
             // Connection OK
-            m_connecStatusLblIcon->setIcon(QIcon(QLatin1String(":/icons/skin/connected.png")));
+            m_connecStatusLblIcon->setIcon(QIcon(QLatin1String(":/icons/skin/connected.svg")));
             m_connecStatusLblIcon->setToolTip(QLatin1String("<b>") + tr("Connection Status:") + QLatin1String("</b><br>") + tr("Online"));
         }
         else {
-            m_connecStatusLblIcon->setIcon(QIcon(QLatin1String(":/icons/skin/firewalled.png")));
+            m_connecStatusLblIcon->setIcon(QIcon(QLatin1String(":/icons/skin/firewalled.svg")));
             m_connecStatusLblIcon->setToolTip(QLatin1String("<b>") + tr("Connection status:") + QLatin1String("</b><br>") + QLatin1String("<i>") + tr("No direct connections. This may indicate network configuration problems.") + QLatin1String("</i>"));
         }
     }
@@ -209,14 +209,14 @@ void StatusBar::updateSpeedLabels()
     QString speedLbl = Utils::Misc::friendlyUnit(sessionStatus.payloadDownloadRate, true);
     int speedLimit = BitTorrent::Session::instance()->downloadSpeedLimit();
     if (speedLimit)
-        speedLbl += " [" + Utils::Misc::friendlyUnit(speedLimit, true) + "]";
-    speedLbl += " (" + Utils::Misc::friendlyUnit(sessionStatus.totalPayloadDownload) + ")";
+        speedLbl += " [" + Utils::Misc::friendlyUnit(speedLimit, true) + ']';
+    speedLbl += " (" + Utils::Misc::friendlyUnit(sessionStatus.totalPayloadDownload) + ')';
     m_dlSpeedLbl->setText(speedLbl);
     speedLimit = BitTorrent::Session::instance()->uploadSpeedLimit();
     speedLbl = Utils::Misc::friendlyUnit(sessionStatus.payloadUploadRate, true);
     if (speedLimit)
-        speedLbl += " [" + Utils::Misc::friendlyUnit(speedLimit, true) + "]";
-    speedLbl += " (" + Utils::Misc::friendlyUnit(sessionStatus.totalPayloadUpload) + ")";
+        speedLbl += " [" + Utils::Misc::friendlyUnit(speedLimit, true) + ']';
+    speedLbl += " (" + Utils::Misc::friendlyUnit(sessionStatus.totalPayloadUpload) + ')';
     m_upSpeedLbl->setText(speedLbl);
 }
 
