@@ -29,7 +29,6 @@
 #include "tagfiltermodel.h"
 
 #include <QDebug>
-#include <QHash>
 #include <QIcon>
 
 #include "base/bittorrent/session.h"
@@ -40,16 +39,16 @@ namespace
 {
     QString getSpecialAllTag()
     {
-        static const QString *const ALL_TAG = new QString(" ");
-        Q_ASSERT(!BitTorrent::Session::isValidTag(*ALL_TAG));
-        return *ALL_TAG;
+        const QString ALL_TAG = QLatin1String(" ");
+        Q_ASSERT(!BitTorrent::Session::isValidTag(ALL_TAG));
+        return ALL_TAG;
     }
 
     QString getSpecialUntaggedTag()
     {
-        static const QString *const UNTAGGED_TAG = new QString("  ");
-        Q_ASSERT(!BitTorrent::Session::isValidTag(*UNTAGGED_TAG));
-        return *UNTAGGED_TAG;
+        const QString UNTAGGED_TAG = QLatin1String("  ");
+        Q_ASSERT(!BitTorrent::Session::isValidTag(UNTAGGED_TAG));
+        return UNTAGGED_TAG;
     }
 }
 
@@ -113,7 +112,7 @@ bool TagFilterModel::isSpecialItem(const QModelIndex &index)
 
 QVariant TagFilterModel::data(const QModelIndex &index, int role) const
 {
-    if (!index.isValid() || index.column() != 0)
+    if (!index.isValid() || (index.column() != 0))
         return QVariant();
 
     const int row = index.internalId();
@@ -136,7 +135,7 @@ QVariant TagFilterModel::data(const QModelIndex &index, int role) const
 Qt::ItemFlags TagFilterModel::flags(const QModelIndex &index) const
 {
     if (!index.isValid())
-        return 0;
+        return Qt::NoItemFlags;
     return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
 }
 
@@ -214,7 +213,7 @@ void TagFilterModel::torrentTagAdded(BitTorrent::TorrentHandle *const torrent, c
     emit dataChanged(i, i);
 }
 
-void TagFilterModel::torrentTagRemoved(BitTorrent::TorrentHandle* const torrent, const QString &tag)
+void TagFilterModel::torrentTagRemoved(BitTorrent::TorrentHandle *const torrent, const QString &tag)
 {
     Q_ASSERT(torrent->tags().count() >= 0);
     if (torrent->tags().count() == 0)
