@@ -1,6 +1,6 @@
 /*
- * Bittorrent Client using Qt4 and libtorrent.
- * Copyright (C) 2006-2012  Christophe Dumez
+ * Bittorrent Client using Qt and libtorrent.
+ * Copyright (C) 2006-2012  Christophe Dumez <chris@qbittorrent.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,8 +24,6 @@
  * modify file(s), you may extend this exception to your version of the file(s),
  * but you are not obligated to do so. If you do not wish to do so, delete this
  * exception statement from your version.
- *
- * Contact : chris@qbittorrent.org
  */
 
 #include "torrentcontentmodelfolder.h"
@@ -43,7 +41,7 @@ TorrentContentModelFolder::TorrentContentModelFolder(const QString &name, Torren
 }
 
 TorrentContentModelFolder::TorrentContentModelFolder(const QList<QVariant> &data)
-    : TorrentContentModelItem(0)
+    : TorrentContentModelItem(nullptr)
 {
     Q_ASSERT(data.size() == NB_COL);
     m_itemData = data;
@@ -82,7 +80,7 @@ void TorrentContentModelFolder::appendChild(TorrentContentModelItem *item)
 
 TorrentContentModelItem *TorrentContentModelFolder::child(int row) const
 {
-    return m_childItems.value(row, 0);
+    return m_childItems.value(row, nullptr);
 }
 
 TorrentContentModelFolder *TorrentContentModelFolder::childFolderWithName(const QString &name) const
@@ -90,7 +88,7 @@ TorrentContentModelFolder *TorrentContentModelFolder::childFolderWithName(const
     foreach (TorrentContentModelItem *child, m_childItems)
         if ((child->itemType() == FolderType) && (child->name() == name))
             return static_cast<TorrentContentModelFolder *>(child);
-    return 0;
+    return nullptr;
 }
 
 int TorrentContentModelFolder::childCount() const
@@ -165,7 +163,7 @@ void TorrentContentModelFolder::recalculateAvailability()
     qreal tAvailability = 0;
     qulonglong tSize = 0;
     bool foundAnyData = false;
-    foreach (TorrentContentModelItem* child, m_childItems) {
+    foreach (TorrentContentModelItem *child, m_childItems) {
         if (child->priority() == prio::IGNORED)
             continue;
 
